/*
 * <header and disclaimer>
 *
 *
 *
 */

#include "serial_lib.h"


static void SIGIO_handler(int status)
{
	/* new line is available to be read */
	newline = TRUE;
}

/* 
 * Function:
 *   TTY_Setup()
 * Parameters:
 *   tty - Interface handle
 * Description:
 *   Configures the interface and sets up a signal handler to respond to inputs.
 * Returns:
 *   always 1
 */
int TTY_Setup(int tty)
{
	struct termios tio;
	struct sigaction sio;

	memset(&tio, 0, sizeof(tio));

	/* 
	 * BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
	 * CRTSCTS : output hardware flow control (only used if the cable has
	 *           all necessary lines. See sect. 7 of Serial-HOWTO)
	 * CS8     : 8N1 (8 databits,No parity,1 Stopbit)
	 * CLOCAL  : Local connection, no modem contol
	 * CREAD   : Enable receiving characters
	 * ICRNL   : Convert carriage return to new line characters
	 * ICANON  : Enable canonical input
	 * ECHO    : Echo input characters
	 */

	tio.c_cflag = B115200 | CS8 | CLOCAL | CREAD;
	tio.c_iflag = ICRNL;
	tio.c_lflag = ICANON | ECHO;

	/* setup and add signal handler to detect */
	/* when the user wrote into the serial interface */
	sigaddset(&sio.sa_mask, SIGIO);
	sio.sa_handler = SIGIO_handler;
	sio.sa_flags = 0;
	sigaction(SIGIO, &sio, NULL);

	/* Set the interface to asynchronous mode */
	fcntl(tty, F_SETOWN, getpid());
	fcntl(tty, F_SETFL, FASYNC);

	tcflush(tty, TCIFLUSH);
	tcsetattr(tty, TCSANOW, &tio);

	return 1;
}

/* 
 * Function:
 *   TTY_Init()
 * Parameters:
 *   *IF - Pointer to path of the tty interface
 * Description:
 *   Opens the interface and calls TTY_Setup for configuration
 * Returns:
 *   1 on success, -1 on error
 */
int TTY_Init(char *IF)
{
	tty = open(IF, O_RDWR | O_NOCTTY);
	if (0 > tty)
	{
		/* In case of an error: print message and leave function */
		printf("Cannot open interface file.\n");
		return -1;
	}

	return TTY_Setup(tty);
}

/* 
 * Function:
 *   TTY_Read()
 * Parameters:
 *   tty - Interface handle
 *   len - Amount of bytes to be read
 * Description:
 *   Waits until a new line of text can be read from the interface
 *   It then reads len bytes into a buffer and returns a pointer
 *   to the beginning of the buffer.
 * Returns:
 *   Pointer to the text on success, NULL on error
 */
char *TTY_Read(int tty, int len)
{
	char *buf;
	int ret;

	buf = malloc(len);
	tcflush(tty, TCIFLUSH);
	newline = FALSE;

	while (!newline)
	{
	}

	ret = read(tty, buf, len);

	if (-1 == ret)
	{
		printf("Error reading interface\n");
		return NULL;
	}

	return buf;
}

/* 
 * Function:
 *   TTY_Write()
 * Parameters:
 *   tty - Interface handle
 *   buf - pointer to the text to be written
 *   len - length of text to be written
 * Description:
 *   Writes text onto the interface
 * Returns:
 *   1 on success, -1 on error
 */
int TTY_Write(int tty, void *buf, int len)
{
	int ret;

	ret = write(tty, buf, len);

	if (-1 == ret)
	{
		printf("Error writing interface\n");
		return -1;
	}

	return 1;
}

/* 
 * Function:
 *   TTY_Close()
 * Parameters:
 *   tty - Interface handle
 * Description:
 *   Closes the interface
 * Returns:
 *   1
 */
int TTY_Close(int tty)
{
	close(tty);

	return 1;
}
